﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;

namespace DssApiSamples.Core
{
    /// <summary>
    /// Discovers all the code samples available to the host application.
    /// </summary>
    public static class CoreUtility
    {
        /// <summary>
        /// Gets the title for an application host.
        /// </summary>
        public static string GetHostTitle(Assembly hostAssembly)
        {
            var assemblyTitle = (AssemblyTitleAttribute) Attribute.GetCustomAttribute(hostAssembly,
                                                                                      typeof (AssemblyTitleAttribute));
            return assemblyTitle.Title + ", version " + hostAssembly.GetName().Version;
        }

        /// <summary>
        /// Gets all the samples available to the host application.
        /// </summary>
        /// <returns>
        /// A list of <see cref="SampleRunner"/> objects.
        /// </returns>
        public static IEnumerable<SampleRunner> GetSamples()
        {
            return 
                //find all concrete classes in all available assemblies that implement ISample
                from a in GetAssemblies()
                from t in a.GetExportedTypes()
                where
                    t.IsClass
                    && !t.IsAbstract
                    && typeof (ISample).IsAssignableFrom(t)
                
                //order by full type name so they show up organized in the host application
                orderby t.FullName
                
                //create SampleRunner instances from those types
                select new SampleRunner(t);
        }

        private static IEnumerable<Assembly> GetAssemblies()
        {
            //look for all DLL's in AppDomain's current directory
            foreach (var file in new DirectoryInfo(AppDomain.CurrentDomain.BaseDirectory).GetFiles("*.dll"))
            {
                Assembly assembly = null;

                //attempt to load file as an assembly
                try
                {
                    assembly = Assembly.LoadFile(file.FullName);
                }
                //file was not an assembly; ignore
                catch (BadImageFormatException) { }

                if (assembly != null) 
                    yield return assembly;
            }
        }
    }
}
