using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace DssApiSamples.Core
{
    /// <summary>
    /// Identifies and runs a single sample.
    /// </summary>
    public class SampleRunner
    {
        private readonly Type _sampleType;

        /// <summary>
        /// Initalizes a new instance of a <see cref="SampleRunner"/> class.
        /// </summary>
        /// <param name="sampleType">
        /// The <see cref="Type"/> of the sample.
        /// </param>
        public SampleRunner(Type sampleType)
        {
            _sampleType = sampleType;

            Name = _sampleType.Name;

            Categories = _sampleType.Namespace.Split(new[] {'.'}).Skip(1).ToList();

            const string delimiter = ".";
            FullName = string.Join(delimiter, Categories.ToArray()) + delimiter + Name;
        }
        
        /// <summary>
        /// Gets the name of the sample.
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// Gets a list of parent categories of whome the sample is a member, starting with the root category.
        /// </summary>
        public IList<string> Categories { get; private set; }

        /// <summary>
        /// Gets the full name of the sample.
        /// </summary>
        public string FullName { get; private set; }

        /// <summary>
        /// Runs the sample, writing a standard messages to the <paramref name="writer"/> 
        /// before and after executing the sample.
        /// </summary>
        /// <param name="writer">
        /// An <see cref="IMessageWriter"/> instance to pass to the <see cref="ISample"/> instance.
        /// </param>
        public void Run(IMessageWriter writer)
        {
            writer.WriteLine("RUNNING: " + FullName + ":");

            var sample = (ISample)Activator.CreateInstance(_sampleType);

            var stopwatch = new Stopwatch();
            stopwatch.Start();
            sample.Execute(writer);
            stopwatch.Stop();

            writer.WriteLine("(Elapsed time: {0})", stopwatch.Elapsed);
            writer.WriteLine();
        }
    }
}
