﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.ServiceModel;
using System.Windows.Forms;
using DssApiSamples.Core;
using DssApiSamples.GuiRunner.Async;
using DssApiSamples.Samples;

namespace DssApiSamples.GuiRunner
{
    public partial class MainForm : Form
    {
        private const string OutputDirectoryName = "Out";
        private const string RunSamplesButtonText = "Run Sample(s)";

        private readonly IMessageWriter _writer;
        private readonly Dictionary<SampleRunner, TreeNode> _sampleNodesBySample = new Dictionary<SampleRunner, TreeNode>();

        public MainForm()
        {
            InitializeComponent();
            _writer = new GuiMessageWriter(_backgroundWorker);
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            Text = CoreUtility.GetHostTitle(Assembly.GetExecutingAssembly());
            _apiUrlToolStripStatusLabel.Text = "API URL: " + ApiUtility.ApiUrl;
            _runCancelButton.Text = RunSamplesButtonText;

            BindSampleList();
        }

        private void BindSampleList()
        {
            //get all samples and populate the tree view
            foreach (var sample in CoreUtility.GetSamples())
            {
                var nodes = _sampleTreeView.Nodes;
                
                //add nodes for the sample's parent categories
                foreach (var category in sample.Categories)
                {
                    var categoryNode = nodes.ContainsKey(category) 
                        ? nodes[category] 
                        : nodes.Add(category, category);
                    AddSampleToNode(categoryNode, sample);
                    nodes = categoryNode.Nodes;
                }

                //add node for the sample itself
                var sampleNode = nodes.Add(sample.Name);

                AddSampleToNode(sampleNode, sample);

                //register sample node in dictionary so node can be found by sample
                _sampleNodesBySample.Add(sample, sampleNode);
            }

            _sampleTreeView.CollapseAll();
            var rootNode = _sampleTreeView.Nodes[0];
            _sampleTreeView.SelectedNode = rootNode;
            rootNode.Expand();
            
        }

        private static void AddSampleToNode(TreeNode node, SampleRunner sample)
        {
            //store samples associated with the node in a list within the node's tag
            var samples = node.Tag == null
                              ? new List<SampleRunner>()
                              : (List<SampleRunner>) node.Tag;
            samples.Add(sample);
            node.Tag = samples;
        }

        private void _runCancelButton_Click(object sender, EventArgs e)
        {
            OnRunCancel();
        }

        private void OnRunCancel()
        {
            if (_backgroundWorker.IsBusy)
            {
                _runCancelButton.Text = "Cancelling...";
                _runCancelButton.Enabled = false;

                //signal to stop current sample run
                _backgroundWorker.CancelAsync();
            }
            else
            {
                //run samples
                if (_sampleTreeView.SelectedNode == null) return;

                _writeToFileCheckBox.Enabled = false;
                _outputFilesLink.Enabled = false;
                _sampleTreeView.Enabled = false;
                _sampleTreeView.HideSelection = true;

                _resultsTextBox.Clear();

                _runCancelButton.Text = "Cancel";

                //execute samples asynchronously so UI remains responsive
                var args = new RunSamplesAsyncArgs
                {
                    Samples = (IEnumerable<SampleRunner>)_sampleTreeView.SelectedNode.Tag
                };
                _backgroundWorker.RunWorkerAsync(args);
            }
        }

        private void _backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            var args = (RunSamplesAsyncArgs) e.Argument;

            if (_writeToFileCheckBox.Checked)
            {
                const string outputFileExtension = ".txt";

                var directory = new DirectoryInfo(OutputDirectoryName);
                if (directory.Exists)
                    //delete existing output files in output directory
                    foreach (var file in directory.GetFiles("*" + outputFileExtension))
                        file.Delete();
                else
                    //create new output directory
                    directory.Create();

                foreach (var sample in args.Samples)
                {
                    //create an output file for each sample
                    var fileName = sample.FullName + outputFileExtension;
                    var filePath = Path.Combine(OutputDirectoryName, fileName);
                    using (var file = File.CreateText(filePath))
                    {
                        var fileWriter = new FileMessageWriter(file, _writer);
                        RunSample(sample, fileWriter);
                    }

                    //exit if cancel was signaled
                    if (HandleCancellation(sample)) return;
                }
            }
            else
                foreach (var sample in args.Samples)
                {
                    RunSample(sample, _writer);

                    //exit if cancel was signaled
                    if (HandleCancellation(sample)) return;
                }
        }

        private bool HandleCancellation(SampleRunner sample)
        {
            if (_backgroundWorker.CancellationPending)
            {
                //send notification to select the current sample node
                SendNotification(
                    new SelectSampleNodeNotification { Sample = sample },
                    false);
                return true;
            }
            return false;
        }

        private void SendNotification(object notification, bool cancel)
        {
            if (cancel)
                _backgroundWorker.CancelAsync();

            _backgroundWorker.ReportProgress(0, notification);
        }

        private void RunSample(SampleRunner sample, IMessageWriter writer)
        {
            SendNotification(
                new BeforeSampleRunNotification {Sample = sample}, false);

            try
            {
                //run sample
                sample.Run(writer);
            }
            catch (ClientErrorException ex)
            {
                //display details
                writer.WriteLine();
                writer.WriteLine("CLIENT ERROR:");
                writer.WriteLine("  " + ex.Message);
                writer.WriteLine("  ErrorCode = {0}", ex.Error.ErrorCode);
                writer.WriteLine("  Messages =");
                foreach (var message in ex.Error.Messages)
                    writer.WriteLine("  " + message);

                //notify user
                SendNotification(
                    new ApiErrorNotification
                        {
                            Message =
                                "An API client error occured!" + Environment.NewLine +
                                "See the sample output for details."
                        }, true);
            }
            catch (ServerErrorException ex)
            {
                //display details
                writer.WriteLine();
                writer.WriteLine("SERVER ERROR:");
                writer.WriteLine("  " + ex.Message);
                writer.WriteLine("  ErrorInstanceId = {0}", ex.Error.ErrorInstanceId);
                writer.WriteLine("  Host = {0}", ex.Error.Host);

                //notify user
                SendNotification(
                    new ApiErrorNotification
                        {
                            Message =
                                "An API server error occured!" + Environment.NewLine +
                                "See the sample output for details."
                        }, true);
            }
            catch (InvalidOutputFieldsException ex)
            {
                //display details
                writer.WriteLine();
                writer.WriteLine("INVALID OUTPUT FIELDS:");
                writer.WriteLine("  " + ex.Message);
                foreach (var field in ex.Fields)
                    writer.WriteLine("  " + field);

                //notify user
                SendNotification(
                    new ApiErrorNotification
                        {
                            Message =
                                "Invalid output fields!" + Environment.NewLine +
                                "See the sample output for details."
                        }, true);
            }
            catch (FaultException ex)
            {
                //display details
                writer.WriteLine();
                writer.WriteLine("GENERAL SOAP FAULT:");
                writer.WriteLine("  " + ex.Message);
                writer.WriteLine("  Fault Code = {0}", ex.Code.Name);

                //notify user
                SendNotification(
                    new ApiErrorNotification
                    {
                        Message =
                            "A general SOAP fault has occurred!" + Environment.NewLine +
                            "See the sample output for details."
                    }, true);
            }
            finally
            {
                SendNotification(
                    new AfterSampleRunNotification {Sample = sample}, false);
            }
        }

        private void _backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            var outputMessageNotification = e.UserState as OutputMessageNotification;
            if (outputMessageNotification != null)
            {
                _resultsTextBox.AppendText(outputMessageNotification.Text);
                return;
            }

            var beforeSampleRunNotification = e.UserState as BeforeSampleRunNotification;
            if (beforeSampleRunNotification != null)
            {
                var node = _sampleNodesBySample[beforeSampleRunNotification.Sample];
                node.EnsureVisible();
                node.ForeColor = Color.Blue;
                return;
            }

            var afterSampleRunNotification = e.UserState as AfterSampleRunNotification;
            if (afterSampleRunNotification != null)
            {
                var node = _sampleNodesBySample[afterSampleRunNotification.Sample];
                node.ForeColor = Color.Empty;
                return;
            }

            var apiErrorNotification = e.UserState as ApiErrorNotification;
            if (apiErrorNotification != null)
            {
                MessageBox.Show(this,
                                apiErrorNotification.Message,
                                "GUI Runner",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
                return;
            }

            var selectSampleNodeNotification = e.UserState as SelectSampleNodeNotification;
            if (selectSampleNodeNotification != null)
            {
                var node = _sampleNodesBySample[selectSampleNodeNotification.Sample];
                _sampleTreeView.SelectedNode = node;
                return;
            }
        }

        private void _backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            //rethrow any unhandled exception that occurred during the sample run
            if (e.Error != null)
                throw e.Error;

            _runCancelButton.Text = RunSamplesButtonText;
            _runCancelButton.Enabled = true;
            _writeToFileCheckBox.Enabled = true;
            _outputFilesLink.Enabled = true;
            _sampleTreeView.Enabled = true;
            _sampleTreeView.HideSelection = false;
        }

        private void outputFilesLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            var directory = new DirectoryInfo(OutputDirectoryName);
            if (directory.Exists)
                Process.Start("explorer.exe", directory.FullName);
            else
                MessageBox.Show(this,
                    "The output directory does not yet exist.\n" + Environment.NewLine +
                    "Run at least one sample with the 'Write output to files' checkbox checked.", 
                    "GUI Runner", 
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Exclamation);
        }
    }
}
