﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.Text;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Advanced
{
    public class RequestExecutionSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    writer.WriteLine("Performing a call to the API using the ApiUtility.Execute method");

                    // Create and initialize the request object. 
                    var request = new GetVersionRequest();
                    
                    //Wrap the call to the API using ApiUtility.Execute, 
                    //which removes the need for the try/catch and also calls ApiUtility.Retry
                    var response = ApiUtility.Execute(
                        () => proxy.GetVersion(request));


                    writer.WriteLine("DSS Version = " + response.GetVersionResult.DssVersion);
                    writer.WriteLine("API Version = " + response.GetVersionResult.ApiVersion);

                    // Access the header value via the proxy.
                    writer.WriteLine("X-Request-Execution-Correlation-Id: {0}", proxy.LastRequestId);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}

namespace DssApiSamples.Samples.Service
{
    public partial class ExtractionServiceClient
    {
        public string LastRequestId;

        protected override ExtractionService CreateChannel()
        {
            Endpoint.Behaviors.Add(new MessageInspectorBehavior(this));
            return base.CreateChannel();
        }
    }

    public class MessageInspectorBehavior : IEndpointBehavior
    {
        private ExtractionServiceClient Parent;

        public MessageInspectorBehavior(ExtractionServiceClient parent)
        {
            Parent = parent;
        }

        public void Validate(ServiceEndpoint endpoint)
        {
            
        }

        public void AddBindingParameters(ServiceEndpoint endpoint, BindingParameterCollection bindingParameters)
        {
        }

        public void ApplyDispatchBehavior(ServiceEndpoint endpoint, EndpointDispatcher endpointDispatcher)
        {
        }

        public void ApplyClientBehavior(ServiceEndpoint endpoint, ClientRuntime clientRuntime)
        {
            clientRuntime.MessageInspectors.Add(new MessageInspector(Parent));
        }

    }

    public class MessageInspector : IClientMessageInspector
    {
        private ExtractionServiceClient Parent;

        public MessageInspector(ExtractionServiceClient parent)
        {
            Parent = parent;
        }

        public object BeforeSendRequest(ref Message request, IClientChannel channel)
        {
            return null;
        }

        public void AfterReceiveReply(ref Message reply, object correlationState)
        {
            Parent.LastRequestId = ((HttpResponseMessageProperty)reply.Properties[HttpResponseMessageProperty.Name]).Headers["X-Request-Execution-Correlation-Id"];
        }
    }


}
