﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Advanced
{
    public class RetrySample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    writer.WriteLine("Performing a call to the API using the ApiUtility.Retry method");

                    // Create and initialize the request object. 
                    var request = new GetInstrumentIdentifierTypesRequest
                                      {
                                          CredentialsHeader = credentials
                                      };

                    // Call the API to get the response, handling API errors.
                    GetInstrumentIdentifierTypesResponse response;
                    try
                    {
                        //Call the API using retry by passing original call into ApiUtility. 
                        //Retry as a delegate (shown as a lambda expression below):
                        response = ApiUtility.Retry(
                            () => proxy.GetInstrumentIdentifierTypes(request));

                        //The anonymous method approach also works:
                        //
                        // response = ApiUtility.Retry(delegate
                        //                                {
                        //                                    return proxy.GetInstrumentIdentifierTypes(request);
                        //                                });
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    // Display the response.
                    foreach (var type in response.GetInstrumentIdentifierTypesResult)
                        writer.WriteLine("  {0}: {1}",
                                     type.Value,
                                     type.Description);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}