﻿using System;
using System.Runtime.Serialization;
using System.ServiceModel.Channels;
using System.Xml;
using System.Xml.Serialization;

namespace DssApiSamples.Samples
{
    /// <summary>
    /// An <see cref="XmlObjectSerializer"/> that can deserialize the detail element
    /// contained within a <see cref="MessageFault"/> into a stronly-typed object.
    /// </summary>
    public class ApiFaultDeserializer : XmlObjectSerializer
    {
        private readonly Type _type;

        /// <summary>
        /// Deserializes the detail element within the specified <see cref="MessageFault"/>.
        /// </summary>
        /// <typeparam name="T">
        /// The type of the object to deserialize.
        /// </typeparam>
        /// <param name="fault">
        /// The <see cref="MessageFault"/> containing the detail.
        /// </param>
        /// <returns>
        /// The deserialized detail object, typed as <typeparamref name="T"/>.
        /// </returns>
        public static T Deserialize<T>(MessageFault fault)
        {
            var serializer = new ApiFaultDeserializer(typeof(T));
            return fault.GetDetail<T>(serializer);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ApiFaultDeserializer"/> class.
        /// </summary>
        /// <param name="type">
        /// The type of the object to attempt to deserialize.
        /// </param>
        public ApiFaultDeserializer(Type type)
        {
            _type = type;
        }

        /// <summary>
        /// Overrides the <see cref="XmlObjectSerializer"/> class's 
        /// <see cref="XmlObjectSerializer.ReadObject(XmlDictionaryReader,bool)"/> method in order to 
        /// deserialize the SOAP fault detail.
        /// </summary>
        public override object ReadObject(XmlDictionaryReader reader, bool verifyObjectName)
        {
            var xmlSerializer = new XmlSerializer(_type,
                                                  reader.LookupNamespace(string.Empty));
            var detail = xmlSerializer.Deserialize(reader);
            return detail;
        }

        public override bool IsStartObject(XmlDictionaryReader reader)
        {
            throw new NotImplementedException();
        }

        public override void WriteStartObject(XmlDictionaryWriter writer, object graph)
        {
            throw new NotImplementedException();
        }

        public override void WriteObjectContent(XmlDictionaryWriter writer, object graph)
        {
            throw new NotImplementedException();
        }

        public override void WriteEndObject(XmlDictionaryWriter writer)
        {
            throw new NotImplementedException();
        }
    }
}
