﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Threading;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples
{
    /// <summary>
    /// Contains methods for performing common tasks with the API.
    /// </summary>
    public static class ApiUtility
    {
        /// <summary>
        /// Gets the configured URL of the API, which gets displayed by the host application.
        /// </summary>
        public static string ApiUrl
        {
            get
            {
                using (var proxy = new ExtractionServiceClient())
                {
                    return proxy.Endpoint.Address.Uri.ToString();
                }
            }
        }

        /// <summary>
        /// Creates a <see cref="CredentialsHeader"/> object that can be used with API operations that require
        /// credentials.
        /// </summary>
        public static CredentialsHeader CreateCredentials()
        {
            //get settings data from the application config file
            var settings = ApiSettings.Default;

            //build credentials from those settings
            return new CredentialsHeader
            {
                Username = settings.Username,
                Password = settings.Password
            };
        }

        /// <summary>
        /// Examines the SOAP fault returned by the API and converts its contents into 
        /// a <see cref="ClientErrorException"/> if a client error occurred, 
        /// a <see cref="ServerErrorException"/> if a server error occurred, 
        /// or the original <see cref="FaultException"/> if any other error occurred.
        /// </summary>
        /// <param name="faultException">
        /// The <see cref="FaultException"/> containing the SOAP fault.
        /// </param>
        public static Exception ConvertFaultException(FaultException faultException)
        {
            var fault = faultException.CreateMessageFault();

            if (fault.HasDetail)
            {
                //attempt to deserialize the fault detail into a ClientError or ServerError
                try
                {
                    //SOAP fault is a ClientError
                    if (fault.Code.IsSenderFault)
                    {
                        var clientError = ApiFaultDeserializer.Deserialize<ClientError>(fault);
                        return new ClientErrorException(faultException.Message, clientError);
                    }

                    //SOAP fault is a ServerError
                    if (fault.Code.IsReceiverFault)
                    {
                        var serverError = ApiFaultDeserializer.Deserialize<ServerError>(fault);
                        return new ServerErrorException(faultException.Message, serverError);
                    }
                }
                catch
                {
                    //swallow any failed attempt to convert a SOAP fault into a ClientError or ServerError
                    //and revert to just returning the FaultException
                }
            }

            //SOAP fault is something else, return original fault
            return faultException;
        }

        /// <summary>
        /// Executes the specified expression multiple times until it succeeds or a configured limit is reached.
        /// </summary>
        /// <typeparam name="T">
        /// The type of the return value of the expression.
        /// </typeparam>
        /// <param name="expression">
        /// The expression to execute.
        /// </param>
        /// <returns>
        /// The result of the expression, if execution is successful.
        /// </returns>
        /// <exception cref="MaxRetriesException">
        /// The maximum number of configured retries has been exceeded.
        /// </exception>
        public static T Retry<T>(Func<T> expression)
        {
            //get settings data from the application config file
            var settings = ApiSettings.Default;

            Exception retryException = null;
            var retries = 0;
            while (retries < settings.MaxRetries)
            {
                try
                {
                    return expression();
                }
                catch (Exception ex)
                {
                    retryException = ex;
                    //determine if exception is qualifies for a retry
                    if (IsRetryable(ex))
                    {
                        retries++;
                        Thread.Sleep(settings.RetrySleepTime);
                    }
                    else
                        throw;
                }
            }
            //max retries was reached, throw exception
            throw new MaxRetriesException(
                string.Format("The maximum number of configired retries ({0}) has been exceeded.", settings.MaxRetries),
                retryException);
        }

        private static bool IsRetryable(Exception exception)
        {
            //add custom logic to examine the exception and determine if the expression can be retryable
            //this logic varies, depending on the requirements of the client application

            //example: check for ProtocolException which occurs if the connection is severed before the call to the API completed
            var protocolException = exception as ProtocolException;
            if (protocolException != null)
                return true;

            //exception is not retryable
            return false;
        }

        /// <summary>
        /// Wraps a call to the API in a common set of error handling and retry logic, 
        /// which is the cleanest way to make calls to the API.
        /// </summary>
        /// <typeparam name="T">
        /// The type of the return value of the call to the API.
        /// </typeparam>
        /// <param name="expression">
        /// The API call, as expressed as a delegate.
        /// </param>
        /// <returns>
        /// The result of the call to the API.
        /// </returns>
        public static T Execute<T>(Func<T> expression)
        {
            try
            {
                return Retry(expression);
            }
            catch (FaultException ex)
            {
                throw ConvertFaultException(ex);
            }
        }

        /// <summary>
        /// Serializes the specified list of values into a single line in CSV format.
        /// </summary>
        /// <param name="values">
        /// The values to serialize.
        /// </param>
        /// <returns>
        /// The values, serialized in CSV format.
        /// </returns>
        public static string ToCsvLine(IEnumerable<object> values)
        {
            var csvValues = (from v in values select ToCsvField(v)).ToArray();
            var csvLine = string.Join(",", csvValues);
            return csvLine;
        }

        private static string ToCsvField(object value)
        {
            const string quote = "\"";

            if (value == null || value is EmptyValue)
                return string.Empty;

            return
                //wrap value in quotes
                quote +
                //escape internal double-quotes
                value.ToString().Replace(quote, quote + quote) +
                quote;
        }
    }
}