﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.DataDictionary
{
    public class CoraxSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string capitalChangeEvent = "CAP";
                    const string dividentEvent = "DIV";

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Getting all Corporate Action fields that:");
                    writer.WriteLine("- are associated with Captital Change ({0}) and Dividend ({1}) events",
                                 capitalChangeEvent, dividentEvent);

                    #endregion

                    #region Make data dictionary request

                    // Create and initialize the request object. 
                    var request = new DefineRequest
                                      {
                                          CredentialsHeader = credentials,
                                          request = new DataDictionaryRequestCorax
                                                        {
                                                            //Note: call GetCoraxEvents to see all Corax events supported. 
                                                            CoraxEvents = new[] {capitalChangeEvent, dividentEvent}
                                                        }
                                      };

                    // Call the API to get the response, handling API errors.
                    DefineResponse response;
                    try
                    {
                        response = proxy.Define(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("{0} fields found:", response.DefineResult.Fields.Length);
                    foreach (var field in response.DefineResult.Fields)
                    {
                        writer.WriteLine("  {0}: {1}", field.Name, field.Description);
                        writer.WriteLine("  - Asset Types = {0}", string.Join(", ", field.AssetTypes));
                        writer.WriteLine("  - Corax Events = {0}", string.Join(", ", field.CoraxEvents));
                        writer.WriteLine("  - Bond Schedule Types = {0}", string.Join(", ", field.BondScheduleTypes));
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}