﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Analytics
{
    public class FixedIncomeAnalyticsSample : ISample
    {
        #region ISample Members

        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    CredentialsHeader credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                        {
                            "Asset ID"
                            , "Asset Status"
                            , "Asset SubType"
                            , "Asset SubType Description"
                            , "Asset Type"
                            , "Asset Type Description"
                            , "Benchmark"
                            , "Benchmark Type"
                            , "Benchmark Value"
                            , "Bond Structure"
                            , "Bond Type"
                            , "Bond Type Description"
                            , "Econometric Average Life"
                            , "Econometric Average Life Date"
                            , "Pricing Method Code"
                            , "Pricing Method Value"
                            , "Redemption Date"
                            , "Redemption Event"
                            , "Redemption Type"
                            , "Sink Speed"
                            , "Years To Redemption"
                            , "Yield Type"
                            , "Yield Type Description"
                        };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing a Fixed Income Analytics extraction that:");
                    writer.WriteLine("- Retrieve user-defined price/yield (P/Y) calculations, duration and convexity calculations, spread calculations, ");
                    writer.WriteLine("- and complete cash flow calculations for U.S. government, international, and corporate securities,");
                    writer.WriteLine("- mortgage-backed securities (MBS),");
                    writer.WriteLine("- collateralized mortgage obligations (CMOs),");
                    writer.WriteLine("- asset-backed and whole loan-backed deals, and a wide range of pools.");
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (string outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Search for instruments

                    const string identifier = "912810PS1";

                    writer.WriteLine("Searching for Instruments that:");
                    writer.WriteLine("- have a Cusip of {0}", identifier);

                    var searchRequest =
                        new InstrumentSearchRequestAll
                            {
                                IdentifierType = "CSP",
                                IdentifierSearchValue = identifier,
                                PreferredIdentifierType = "CSP"
                            };

                    ValidatedInstrument[] instruments = ExtractionUtility.SearchForInstruments(searchRequest, proxy,
                                                                                               ref credentials);
                    writer.WriteLine("{0} instruments found:", instruments.Length);
                    foreach (ValidatedInstrument instrument in instruments)
                        writer.WriteLine("  {0}: {1} {2}", instrument.IdentifierType, instrument.IdentifierValue,
                                         instrument.Source);

                    #endregion

                    #region Perform extractions

                    writer.WriteLine("Performing extraction...");

                    // Create and initialize the request object. 
                    var request =
                        new ExtractRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentExtractionRequestFixedIncomeAnalytics
                                    {
                                        // These are required values.
                                        Instruments = instruments,
                                        OutputFields = outputFields
                                    }
                            };

                    // Call the API to get the response, handling API errors.
                    ExtractResponse response;
                    try
                    {
                        response = proxy.Extract(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    ExtractionResponse extractionResponse = response.ExtractResult;
                    writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

                    // Pivot the column-based results from API to row-based results.
                    RowBasedResult rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns,
                                                                                             extractionResponse.RowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    // Display the extraction messages.
                    writer.WriteLine(
                        "Extraction Messages:");
                    foreach (string message in extractionResponse.Messages)
                        writer.WriteLine(message);
                    writer.WriteLine();

                    #endregion

                    // Check the validity of all output fields.
                    ExtractionUtility.EnsureOutputFields(response.ExtractResult);
                }
                    // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }

        #endregion
    }
}