﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Analytics
{
    public class TechnicalIndicatorAnalyticsSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                    {
                        new InstrumentIdentifier
                            {
                                IdentifierType = "RIC",
                                IdentifierValue = "IBM.N"
                            }
                    };

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                                           {
                                               "Alpha - 3 Year Weekly",
                                               "Alpha - 5 Year Monthly",
                                               "Beta - 3 Year Weekly",
                                               "Beta - 5 Year Monthly",
                                               "Average Volume - 10 Days",
                                               "Volatility - 10 days",
                                               "Instrument ID",
                                               "Instrument ID Type",
                                               "Asset ID",
                                               "Asset Type",
                                               "Asset Type Description",
                                               "Country of Incorporation"
                                           };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing a Technical Indicators extraction that:");
                    writer.WriteLine("- Retrieve technical analytical data like Alpha, Beta, ");
                    writer.WriteLine("- Average Volume, Moving Averages, Volatility ");
                    writer.WriteLine("- and Percent Changes based on End of Day Prices. ");
                    writer.WriteLine("- Applies to Global Equity Instruments. ");
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (var outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Validate instruments

                    writer.WriteLine("Validating Instruments...");

                    // Using the instrument identifier(s) defined above, request the validated instruments to be used in the extraction.
                    // NOTE: see Validation.InstrumentSample for more information on how to perform an Instrument validation.
                    var validationRequest =
                        new InstrumentValidationRequest
                        {
                            Identifiers = identifiers
                        };
                    var instruments = ExtractionUtility.ValidateInstruments(validationRequest, proxy, ref credentials);

                    #endregion

                    #region Perform extraction

                    writer.WriteLine("Performing extraction...");

                    // Create and initialize the request object. 
                    var request =
                        new ExtractRequest
                        {
                            CredentialsHeader = credentials,
                            request = new InstrumentExtractionRequestTechnicalIndicators()
                            {
                                // These are required values.
                                OutputFields = outputFields,
                                Instruments = instruments
                            }
                        };

                    // Call the API to get the response, handling API errors.
                    ExtractResponse response;
                    try
                    {
                        response = proxy.Extract(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion


                    #region Display results

                    var extractionResponse = response.ExtractResult;
                    writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns,
                                                                                  extractionResponse.RowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    // Display the extraction messages.
                    writer.WriteLine(
                        "Extraction Messages:");
                    foreach (var message in extractionResponse.Messages)
                        writer.WriteLine(message);
                    writer.WriteLine();

                    #endregion

                    // Check the validity of all output fields.
                    ExtractionUtility.EnsureOutputFields(response.ExtractResult);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}