using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Counterparty
{
    public class AuditSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                                          {
                                              new CounterpartyIdentifier
                                                  {
                                                      IdentifierType = "RCP",
                                                      IdentifierValue = "300012286"
                                                  },
                                              new CounterpartyIdentifier
                                                  {
                                                      IdentifierType = "ORG",
                                                      IdentifierValue = "21528"
                                                  }
                                          };

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                                           {
                                               "RCP ID",
                                               "Audit Modified Date",
                                               "Audit Source",
                                               "Audited Field Name"
                                           };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing an Counterparty Audit extraction that:");
                    writer.WriteLine("- gets data for the following Counterparty identifiers:");
                    foreach (var identifier in identifiers)
                        writer.WriteLine("  {0}: {1}",
                                     identifier.IdentifierType,
                                     identifier.IdentifierValue);
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (var outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Validate instruments

                    writer.WriteLine("Validating Counterparties...");

                    // Using the instrument identifier(s) defined above, request the validated instruments to be used in the extraction.
                    // NOTE: see Validation.InstrumentSample for more information on how to perform an Instrument validation.
                    var validationRequest =
                        new CounterpartyValidationRequest
                            {
                                Identifiers = identifiers
                            };
                    var counterparties = ExtractionUtility.ValidateCounterparties(validationRequest, proxy,
                                                                                  ref credentials);

                    #endregion

                    #region Perform extraction

                    writer.WriteLine("Performing extraction...");

                    // Create and initialize the request object. 
                    var request =
                        new ExtractRequest
                            {
                                CredentialsHeader = credentials,
                                request = new CounterpartyExtractionRequestAudit
                                              {
                                                  // These are required values.
                                                  Counterparties = counterparties,
                                                  OutputFields = outputFields
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    ExtractResponse response;
                    try
                    {
                        response = proxy.Extract(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    var extractionResponse = response.ExtractResult;
                    writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns,
                                                                                  extractionResponse.RowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    // Display the extraction messages.
                    writer.WriteLine(
                        "Extraction Messages:");
                    foreach (var message in extractionResponse.Messages)
                        writer.WriteLine(message);
                    writer.WriteLine();

                    #endregion

                    // Check the validity of all output fields.
                    ExtractionUtility.EnsureOutputFields(response.ExtractResult);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}