using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Estimates
{
    public class DetailSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                                          {
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "KO.N"
                                                  },
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "IBM.N"
                                                  }
                                          };

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                                           {
                                               "RIC",
                                               "Measure",
                                               "Period",
                                               "Period Date",
                                               "Broker Name",
                                               "Analyst Name",
                                               "Estimate",
                                               "Estimate Date"
                                           };
                    const string companyType = "Consolidated";
                    const string epsType = "EPS";
                    const string salesType = "SAL";
                    const string fiscalYearFrom = "FY1";
                    const string fiscalYearTo = "FY2";

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing an Estimate Detail extraction that:");
                    writer.WriteLine("- gets data for the following instrument identifiers:");
                    foreach (var identifier in identifiers)
                        writer.WriteLine("  {0}: {1} {2}",
                                     identifier.IdentifierType,
                                     identifier.IdentifierValue,
                                     string.IsNullOrEmpty(identifier.Source) ? null : "(" + identifier.Source + ")");
                    writer.WriteLine("- only gets data for {0} companies:", companyType);
                    writer.WriteLine("- gets Earnings Per Share ({0}) and Sales ({1}) estimates:", epsType, salesType);
                    writer.WriteLine("- gets estimates for the current fiscal year to the next fiscal year ({0} to {1}):",
                                 fiscalYearFrom, fiscalYearTo);
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (var outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Validate instruments

                    writer.WriteLine("Validating instruments...");

                    // Using the instrument identifier(s) defined above, request the validated instruments to be used in the extraction.
                    // NOTE: see Validation.InstrumentSample for more information on how to perform an Instrument validation.
                    var validationRequest =
                        new InstrumentValidationRequest
                            {
                                Identifiers = identifiers
                            };
                    var instruments = ExtractionUtility.ValidateInstruments(validationRequest, proxy, ref credentials);

                    #endregion

                    #region Perform extractions

                    writer.WriteLine("Performing extraction...");

                    // Create and initialize the request object.
                    var request =
                        new ExtractRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentExtractionRequestEstimateDetail
                                              {
                                                  // These are required values.
                                                  Instruments = instruments,
                                                  OutputFields = outputFields,
                                                  // NOTE: call GetEstimateCompanyTypes to see all supported company types.
                                                  CompanyType = companyType,
                                                  // NOTE: call GetEstimateTypes to see all estimate types supported by a given Estimate Extraction type.
                                                  EstimateTypes = new[] {epsType, salesType},
                                                  // NOTE: call GetEstimateHistoricalRelativeFiscalYearValues to see fiscal-year values.
                                                  TimePeriodConstraint = new EstimateTimePeriodConstraintRelative
                                                                             {
                                                                                 FiscalYearFrom = fiscalYearFrom,
                                                                                 FiscalYearTo = fiscalYearTo
                                                                             }
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    ExtractResponse response;
                    try
                    {
                        response = proxy.Extract(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    var extractionResponse = response.ExtractResult;
                    writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns,
                                                                                  extractionResponse.RowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    // Display the extraction messages.
                    writer.WriteLine(
                        "Extraction Messages:");
                    foreach (var message in extractionResponse.Messages)
                        writer.WriteLine(message);
                    writer.WriteLine();

                    #endregion

                    // Check the validity of all output fields.
                    ExtractionUtility.EnsureOutputFields(response.ExtractResult);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}