﻿using System.Linq;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Estimates
{
    public class SummaryWithLargeListSample
        : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                                          {
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "CHR",
                                                      // This chained RIC, when expanded, will return the S&P 100 equities.
                                                      IdentifierValue = "0#.OEXA"
                                                  }
                                          };

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                                           {
                                               "RIC",
                                               "Measure",
                                               "Period",
                                               "Period Date",
                                               "Number of Estimates",
                                               "High Estimate",
                                               "Low Estimate",
                                               "Mean Estimate",
                                               "Median Estimate"
                                           };
                    const string companyType = "Consolidated";
                    const string epsType = "EPS";
                    const string salesType = "SAL";
                    const string fiscalYearFrom = "FY1";
                    const string fiscalYearTo = "FY3";

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing an Estimate Summary extraction that:");
                    writer.WriteLine("- gets data for all of the equities in the S&P 100 index");
                    writer.WriteLine("- only gets data for {0} companies", companyType);
                    writer.WriteLine("- gets Earnings Per Share ({0}) and Sales ({1}) estimates:", epsType, salesType);
                    writer.WriteLine(
                        "- gets estimates for the current fiscal year to the next two fiscal years ({0} to {1}):",
                        fiscalYearFrom, fiscalYearTo);
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (var outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Validate instruments

                    writer.WriteLine("Validating instruments, performing expansion...");

                    // Using the instrument identifier(s) defined above, request the validated instruments to be used in the extraction.  
                    // NOTE: see Validation.InstrumentSample for more information on how to perform an Instrument validation
                    var validationRequest =
                        new InstrumentValidationRequest
                            {
                                Identifiers = identifiers,
                                PerformExpansion = true
                            };

                    var instruments = ExtractionUtility.ValidateInstruments(validationRequest, proxy, ref credentials);
                    writer.WriteLine("{0} constituent RIC's returned:", instruments.Length);
                    foreach (var instrument in instruments)
                        writer.WriteLine("  {0} ({1})", instrument.Ric, instrument.Source);

                    #endregion

                    #region Get max instruments per extraction

                    writer.WriteLine("Calling the GetInstrumentExtractionTypes discoverability method");
                    writer.WriteLine(
                        "to dynamically determine the maximum number of instruments that can be passed to a Estimate Summary extraction...");

                    var discoRequest =
                        new GetInstrumentExtractionTypesRequest
                            {
                                CredentialsHeader = credentials
                            };

                    // Call API using ApiUtility.Execute helper method.
                    var discoResponse = ApiUtility.Execute(
                        () => proxy.GetInstrumentExtractionTypes(discoRequest));

                    var maxInstruments = discoResponse.GetInstrumentExtractionTypesResult
                        .Single(i => i.Value == "EST")
                        .MaximumInstrumentsAllowed;

                    writer.WriteLine("The maximum number of instruments that can be passed is: {0}", maxInstruments);

                    #endregion

                    #region Build reusable extraction request

                    var extractionRequest =
                        new InstrumentExtractionRequestEstimateSummary
                            {
                                // These are required values.
                                //Instrument will be set during each loop to Extract 
                                OutputFields = outputFields,
                                // NOTE: call GetEstimateCompanyTypes to see all supported company types.
                                CompanyType = companyType,
                                // NOTE: call GetEstimateTypes to see all estimate types supported by a given Estimate Extraction type.
                                EstimateTypes = new[] {epsType, salesType},
                                // NOTE: call GetEstimateHistoricalRelativeFiscalYearValues to see fiscal-year values.
                                TimePeriodConstraint = new EstimateTimePeriodConstraintRelative
                                                           {
                                                               FiscalYearFrom = fiscalYearFrom,
                                                               FiscalYearTo = fiscalYearTo
                                                           }
                            };
                    var extractRequest =
                        new ExtractRequest
                            {
                                CredentialsHeader = credentials,
                                request = extractionRequest
                            };

                    #endregion

                    #region Perform extractions

                    writer.WriteLine("Performing extraction(s)...");

                    // Loop through all instruments, sending them in batches to the API.
                    ExtractionColumn[] combinedColumns = null;
                    var totalRowCount = 0;
                    var batches = ExtractionUtility.BatchInstruments(instruments, maxInstruments);
                    foreach (var batch in batches)
                    {
                        writer.WriteLine("Extracting batch of {0} instrument(s)...", maxInstruments);
                        extractionRequest.Instruments = batch;

                        // Call API using ApiUtility.Execute helper method.
                        var extractResponse = ApiUtility.Execute(
                            () => proxy.Extract(extractRequest));
                        var extractionResponse = extractResponse.ExtractResult;

                        // Do not bother appending row values if now rows were returned.
                        if (extractionResponse.RowCount <= 0) continue;

                        writer.WriteLine("Rows returned: {0}", extractionResponse.RowCount);
                        totalRowCount += extractionResponse.RowCount;

                        if (combinedColumns == null)
                            // Since this is the first batch, we initialize columns.
                            combinedColumns = extractionResponse.Columns;
                        else
                        {
                            // Subsequent batch: append values in extraction response to columns.
                            for (var i = 0; i < extractionResponse.Columns.Length; i++)
                            {
                                // Get the column from both the current extraction response and the combined column.
                                var column = extractionResponse.Columns[i];
                                var combinedColumn = combinedColumns[i];

                                // Append the values of the current extraction response column to the combined column.
                                combinedColumn.Values = combinedColumn.Values.Concat(column.Values).ToArray();
                            }
                        }
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("Total Row Count = {0}", totalRowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotExtractionResult(combinedColumns, totalRowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}