﻿using System.Collections.Generic;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Other
{
    public class FundAllocationSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                        {
                            "Allocation Date"
                            ,"Allocation Item"
                            ,"Allocation Percentage"
                            ,"Allocation Rank"
                            ,"Allocation Type"
                            ,"Asset Type"
                            ,"Asset Type Description"
                            ,"CUSIP"
                            ,"FundSERVAC"
                            ,"FundSERVBE"
                            ,"FundSERVDM"
                            ,"FundSERVDO"
                            ,"FundSERVFE"
                            ,"FundSERVIS"
                            ,"FundSERVLL"
                            ,"FundSERVNL"
                            ,"FundSERVVS"
                            ,"Identifier Segment Source"
                            ,"Identifier Segment Source Description"
                            ,"Instrument ID"
                            ,"Instrument ID Type"
                            ,"ISIN"
                            ,"Lipper ID"
                            ,"PILC"
                            ,"RIC"
                            ,"Security Description"
                            ,"SEDOL"
                            ,"Usage Instrument SubType"
                            ,"Usage Instrument Type"
                            ,"User Defined Identifier"
                            ,"User Defined Identifier2"
                            ,"User Defined Identifier3"
                            ,"Valoren"
                            ,"Wertpapier"
                        };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing a Fund Allocation extraction that:");
                    writer.WriteLine("- Retrieves one or more fund allocation schemes, including asset, country, currency, ");
                    writer.WriteLine("- sector and top ten holding, for Lipper Fund instruments.");
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (var outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Search for instruments

                    const string lipperId = "60000012";

                    writer.WriteLine("Searching for Funds that:");
                    writer.WriteLine("- have a LipperID of {0}", lipperId);

                    var searchRequest =
                            new InstrumentSearchRequestAll // TODO: This is temporary. The correct search functionality will be implemented in 9.2 
                            {
                                IdentifierType = "LIP",
                                IdentifierSearchValue = lipperId,
                                PreferredIdentifierType = "LIP"
                            };

                    var instruments = ExtractionUtility.SearchForInstruments(searchRequest, proxy, ref credentials);
                    writer.WriteLine("{0} instruments found:", instruments.Length);
                    foreach (var instrument in instruments)
                        writer.WriteLine("  {0}: {1} {2}", instrument.IdentifierType, instrument.IdentifierValue,
                                     instrument.Source);

                    #endregion

                    #region Perform extractions

                    writer.WriteLine("Performing extraction...");

                    // Create and initialize the request object. 
                    var request =
                        new ExtractRequest
                        {
                            CredentialsHeader = credentials,
                            request = new InstrumentExtractionRequestFundAllocation
                            {
                                // These are required values.
                                Instruments = instruments,
                                OutputFields = outputFields,
                                FieldConstraint = new FundAllocationFieldConstraint()
                                    {
                                        AllocationTypes = new string[] {"ASSET", "CNTRY", "CURR", "INDUS", "HLD10"}
                                    }
                            }
                        };

                    // Call the API to get the response, handling API errors.
                    ExtractResponse response;
                    try
                    {
                        response = proxy.Extract(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    var extractionResponse = response.ExtractResult;
                    writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns,
                                                                                  extractionResponse.RowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    // Display the extraction messages.
                    writer.WriteLine(
                        "Extraction Messages:");
                    foreach (var message in extractionResponse.Messages)
                        writer.WriteLine(message);
                    writer.WriteLine();

                    #endregion

                    // Check the validity of all output fields.
                    ExtractionUtility.EnsureOutputFields(response.ExtractResult);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}