﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Other
{
	public class NewsAnalyticsSample : ISample
	{
		public void Execute(IMessageWriter writer)
		{
			// The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
			using (var proxy = new ExtractionServiceClient())
			{
				// Wrap the proxy call in a try block in order to handle communication exceptions properly. 
				try
				{
					// Credentials are required to authenticate the request.
					var credentials = ApiUtility.CreateCredentials();

					#region Declare inputs

					var identifiers = new[]
                                          {
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "KO.N"
                                                  }
                                          };


					// Identify the desired output fields for this extraction. 
					var outputFields = new[]
                                           {
                                               "RIC",
                                               "Headline",
                                               "News Source",
                                               "Reuters Editorial RIC",
                                               "Story Date Time"
                                           };
                    const int daysLimit = 7;

					#endregion

					#region Display sample description

					writer.WriteLine("Performing a News Analytics extraction that:");
					writer.WriteLine("- gets data for the following instrument identifiers:");
					foreach (var identifier in identifiers)
						writer.WriteLine("  {0}: {1} {2}",
									 identifier.IdentifierType,
									 identifier.IdentifierValue,
									 string.IsNullOrEmpty(identifier.Source) ? null : "(" + identifier.Source + ")");
				    writer.WriteLine("- that have news events that have occurred within the last " + daysLimit + " days.");
                    writer.WriteLine("- returns data for the following output fields:");
					foreach (var outputField in outputFields)
						writer.WriteLine("  " + outputField);


					#endregion

					#region Validate instruments

					// Using the instrument identifier(s) defined above, request the validated instruments to be used in the extraction.
					writer.WriteLine("Validating instruments...");
					// NOTE: see Validation.InstrumentSample for more information on how to perform an Instrument validation
					var validationRequest =
						new InstrumentValidationRequest
						{
							Identifiers = identifiers
						};
					var instruments = ExtractionUtility.ValidateInstruments(validationRequest, proxy, ref credentials);

					#endregion

					#region Perform extractions

					writer.WriteLine("Performing extraction...");

					// Create and initialize the request object. 
					var request =
						new ExtractRequest
							{
								CredentialsHeader = credentials,
								request = new InstrumentExtractionRequestNewsAnalytics
					          	{
					          		// These are required values.
					          		Instruments = instruments,
					         		OutputFields = outputFields,
									QueryConstraint = new NewsAnalyticsQueryConstraintDaysAgoDelta
									{
                                        Days = daysLimit
									}
								}
            				};

					// Call the API to get the response, handling API errors.
					ExtractResponse response;
					try
					{
						response = proxy.Extract(request);
					}
					catch (FaultException ex)
					{
						throw ApiUtility.ConvertFaultException(ex);
					}

					#endregion

					#region Display results

					var extractionResponse = response.ExtractResult;
					writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

					// Pivot the column-based results from API to row-based results.
					var rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns,
																				  extractionResponse.RowCount);

					// Display the results in CSV format.
					writer.WriteLine(
						"Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
					writer.WriteLine();
					writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
					foreach (var row in rowBasedResults.Rows)
						writer.WriteLine(ApiUtility.ToCsvLine(row));
					writer.WriteLine();

					// Display the extraction messages.
					writer.WriteLine(
						"Extraction Messages:");
					foreach (var message in extractionResponse.Messages)
						writer.WriteLine(message);
					writer.WriteLine();

					#endregion

					// Check the validity of all output fields.
					ExtractionUtility.EnsureOutputFields(response.ExtractResult);
				}
				// If any exception occurs, close the communication channel and rethrow the exception.
				catch
				{
					proxy.Abort();
					throw;
				}
			}
		}

	}
}
