﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Other
{
    public class TermsAndConditionsSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient()) { 

 			 // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
 			try
            {
                // Credentials are required to authenticate the request.
                var credentials = ApiUtility.CreateCredentials();

                #region Declare inputs

                // Identify the desired output fields for this extraction. 
                var outputFields = new[]
                                   {
                                       "CUSIP",
                                       "Coupon Rate",
                                       "Coupon Frequency",
                                       "Maturity Date",
                                       "Moodys Rating"
                                   };

                #endregion

                #region Display sample description

                writer.WriteLine("Performing a Terms and Conditions extraction that:");
                writer.WriteLine("- gets its instruments from a Gov/Corp search");
                writer.WriteLine("- returns data for the following output fields:");
                foreach (var outputField in outputFields)
                    writer.WriteLine("  " + outputField);

                const string spType = "AAA";
                const string country = "US";

                writer.WriteLine("Searching for Gov/Corp Instruments that:");
                writer.WriteLine("- are corporate bonds");
                writer.WriteLine("- have a S&P rating of {0}", spType);
                writer.WriteLine("- are from the United States ({0})", country);
                writer.WriteLine("- are callable");
                writer.WriteLine("- are putable");

                #endregion

                #region Search instruments

                // NOTE: see Samples.Search.Instrument.GovCorpSample for more information on how to perform a Gov/Corp search.
                var searchRequest =
                    new InstrumentSearchRequestGovCorp
                        {
                            IdentifierType = "CSP",
                            IdentifierSearchValue = "*",
                            PreferredIdentifierType = "CSP",
                            IsCorporateGroup = true,
                            StandardPoorsTypes = new[] {spType},
                            CountryType = country,
                            IsCallable = true,
                            IsPutable = true
                        };
                var instruments = ExtractionUtility.SearchForInstruments(searchRequest, proxy, ref credentials);
                writer.WriteLine("{0} instruments found:", instruments.Length);
                foreach (var instrument in instruments)
                    writer.WriteLine("  {0}: {1} {2}", instrument.IdentifierType, instrument.IdentifierValue, instrument.Source);

                #endregion

                #region Perform extractions

                writer.WriteLine("Performing extraction...");

                // Create and initialize the request object. 
                var request =
                    new ExtractRequest
                        {
                            CredentialsHeader = credentials,
                            request = new InstrumentExtractionRequestTermsAndConditions
                                          {
                                              // These are required values.
                                              Instruments = instruments,
                                              OutputFields = outputFields
                                          }
                        };

                // Call the API to get the response, handling API errors.
                ExtractResponse response;
                try
                {
                    response = proxy.Extract(request);
                }
                catch (FaultException ex)
                {
                    throw ApiUtility.ConvertFaultException(ex);
                }

                #endregion

                #region Display results

                var extractionResponse = response.ExtractResult;
                writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

                // Pivot the column-based results from API to row-based results.
                var rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns, extractionResponse.RowCount);

                // Display the results in CSV format.
                writer.WriteLine("Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                writer.WriteLine();
                writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                foreach (var row in rowBasedResults.Rows)
                    writer.WriteLine(ApiUtility.ToCsvLine(row));
                writer.WriteLine();

                // Display the extraction messages.
                writer.WriteLine(
                    "Extraction Messages:");
                foreach (var message in extractionResponse.Messages)
                    writer.WriteLine(message);
                writer.WriteLine();

                #endregion

                // Check the validity of all output fields.
                ExtractionUtility.EnsureOutputFields(response.ExtractResult);
            }
            // If any exception occurs, close the communication channel and rethrow the exception.
            catch
            {
                proxy.Abort();
                throw;
            }
            }
        }
    }
}
