﻿using System;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Pricing
{
    public class PremiumPricingSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                                           {
                                               "CUSIP",
                                               "Issuer Name",                                              
                                               "Maturity Date",                                               
                                               "Previous Close Price",
                                               "Bid Price",
                                               "Ask Price",
                                               "Mid Price"
                                           };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing a 3PM US Premium Pricing extraction that:");
                    writer.WriteLine("- gets its instruments from a Gov/Corp search");
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (var outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Get instruments via search

					const string subGroup = "GBND";
                    const string spType = "AAA";
                    const string country = "US";
                    const string assetStatusType = "ISS";
					const string contributorType = "EJV";
					const int couponLimit = 1;
                    const int matureYears = 2;

                    writer.WriteLine("Searching for Gov/Corp Instruments that:");
                    writer.WriteLine("- are government bonds");
                    writer.WriteLine("- have a sub-group type of 'Notes/Bonds Govt' ({0})", subGroup);
                    writer.WriteLine("- have a S&P rating of {0}", spType);
                    writer.WriteLine("- are from the United States ({0})", country);
                    writer.WriteLine("- are issued ({0})", assetStatusType);
					writer.WriteLine("- have a contributor of Tr Pricing Service ({0})", contributorType);
					writer.WriteLine("- have a coupon value that is greater than {0}", couponLimit);
                    writer.WriteLine("- mature in more than {0} years", matureYears);
                    // NOTE: see Samples.Search.Instrument.GovCorpSample for more information on how to perform a Gov/Corp search
                    var searchRequest =
                        new InstrumentSearchRequestGovCorp
                            {
                                IdentifierType = "RIC",
                                IdentifierSearchValue = "*",
                                PreferredIdentifierType = "RIC",
								IsGovernmentGroup = true,
                                SubGroupType = subGroup,
                                StandardPoorsTypes = new[] {spType},
                                CountryType = country,
                                AssetStatusTypes = new[] {assetStatusType},
								ContributorType = contributorType,
								CouponComparisonOperator = ">",
                                CouponPrimaryValue = couponLimit,
                                MaturityDateComparisonOperator = ">",
                                MaturityDatePrimaryValue = DateTime.Now.AddYears(matureYears)
                            };
                    var instruments = ExtractionUtility.SearchForInstruments(searchRequest, proxy, ref credentials);
                    writer.WriteLine("{0} instruments found:", instruments.Length);
                    foreach (var instrument in instruments)
                        writer.WriteLine("  {0}: {1} {2}", instrument.IdentifierType, instrument.IdentifierValue,
                                     instrument.Source);

                    #endregion

                    #region Perform extraction

                    writer.WriteLine("Performing extraction...");

                    // Create and initialize the request object. 
                    var request =
                        new ExtractRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentExtractionRequestPremiumPricingUS3PM
                                              {
                                                  // These are required values.
                                                  Instruments = instruments,
                                                  OutputFields = outputFields
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    ExtractResponse response;
                    try
                    {
                        response = proxy.Extract(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    var extractionResponse = response.ExtractResult;
                    writer.WriteLine("Row Count = {0}", extractionResponse.RowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotExtractionResult(extractionResponse.Columns,
                                                                                  extractionResponse.RowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    // Display the extraction messages.
                    writer.WriteLine(
                        "Extraction Messages:");
                    foreach (var message in extractionResponse.Messages)
                        writer.WriteLine(message);
                    writer.WriteLine();

                    #endregion

                    // Check the validity of all output fields.
                    ExtractionUtility.EnsureOutputFields(response.ExtractResult);
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}