﻿using System;
using System.Linq;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Extraction.Pricing
{
    public class TimeSeriesWithLargeListSample
        : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                                          {
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "GOOG.O"
                                                  },
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "IBM.N"
                                                  },
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "KO.N"
                                                  },
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "AAPL.O"
                                                  },
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "TRI.N"
                                                  }
										  };

                    // Identify the desired output fields for this extraction. 
                    var outputFields = new[]
                                           {
                                               "RIC",
                                               "Trade Date",
                                               "Security Description",
                                               "Ask Price",
                                               "Bid Price",
                                               "Ask Yield",
                                               "Bid Yield",
                                               "Ask Discount",
                                               "Bid Discount",
                                               "Close Price",
                                               "High Price",
                                               "Low Price"
                                           };
                    var startDate = DateTime.Now.AddDays(-7);
                    var endDate = DateTime.Now;

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Performing an TimeSeries Pricing extraction that:");
                    writer.WriteLine("- gets data for multiple US Equities");
                    writer.WriteLine("- gets history for the last week ({0:d} - {1:d})", startDate, endDate);
                    writer.WriteLine("- returns data for the following output fields:");
                    foreach (var outputField in outputFields)
                        writer.WriteLine("  " + outputField);

                    #endregion

                    #region Validate instruments 

                    writer.WriteLine("Validating instruments...");
                    // NOTE: see Validation.InstrumentSample for more information on how to perform an Instrument validation
                    var validationRequest =
                        new InstrumentValidationRequest
                            {
                                Identifiers = identifiers,
                                PerformExpansion = false
                            };
                    var instruments = ExtractionUtility.ValidateInstruments(validationRequest, proxy, ref credentials);
                    writer.WriteLine("{0} RIC's returned:", instruments.Length);
                    foreach (var instrument in instruments)
                        writer.WriteLine("  {0} ({1})", instrument.Ric, instrument.Source);

                    #endregion

                    #region Get max instruments per extraction

                    writer.WriteLine("Calling the GetInstrumentExtractionTypes discoverability method");
                    writer.WriteLine(
                        "to dynamically determine the maximum number of instruments that can be passed to a TimeSeries Pricing extraction...");

                    var discoRequest =
                        new GetInstrumentExtractionTypesRequest
                            {
                                CredentialsHeader = credentials
                            };

                    //call API using ApiUtility.Execute helper method
                    var discoResponse = ApiUtility.Execute(
                        () => proxy.GetInstrumentExtractionTypes(discoRequest));

                    var maxInstruments = discoResponse.GetInstrumentExtractionTypesResult
                        .Single(i => i.Value == "TPX")
                        .MaximumInstrumentsAllowed;

                    writer.WriteLine("The maximum number of instruments that can be passed is: {0}", maxInstruments);

                    #endregion

                    #region Build reusable extraction request

                    var extractionRequest =
                        new InstrumentExtractionRequestTimeSeriesPricing
                            {
                                // These are required values.
                                //Instrument will be set during each loop to Extract 
                                OutputFields = outputFields,
                                StartDate = startDate,
                                EndDate = endDate
                            };
                    var extractRequest =
                        new ExtractRequest
                            {
                                CredentialsHeader = credentials,
                                request = extractionRequest
                            };

                    #endregion

                    #region Perform extractions

                    writer.WriteLine("Performing extraction(s)...");

                    //loop through all instruments, sending them in batches to the API 
                    ExtractionColumn[] combinedColumns = null;
                    var totalRowCount = 0;
                    var batches = ExtractionUtility.BatchInstruments(instruments, maxInstruments);
                    foreach (var batch in batches)
                    {
                        writer.WriteLine("Extracting batch of {0} instrument(s)...", maxInstruments);
                        extractionRequest.Instruments = batch;

                        //call API using ApiUtility.Execute helper method
                        var extractResponse = ApiUtility.Execute(
                            () => proxy.Extract(extractRequest));
                        var extractionResponse = extractResponse.ExtractResult;

                        //don't bother appending row values if now rows were returned
                        if (extractionResponse.RowCount <= 0) continue;

                        writer.WriteLine("Rows returned: {0}", extractionResponse.RowCount);
                        totalRowCount += extractionResponse.RowCount;

                        if (combinedColumns == null)
                            //first batch; initialize columns
                            combinedColumns = extractionResponse.Columns;
                        else
                        {
                            //subsequent batch; append values in extraction response to columns
                            for (var i = 0; i < extractionResponse.Columns.Length; i++)
                            {
                                //get the column from both the current extraction response and the combined column
                                var column = extractionResponse.Columns[i];
                                var combinedColumn = combinedColumns[i];

                                //append the values of the current extraction response column to the combined column
                                combinedColumn.Values = combinedColumn.Values.Concat(column.Values).ToArray();
                            }
                        }
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("Total Row Count = {0}", totalRowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotExtractionResult(combinedColumns, totalRowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}