using System.Collections.Generic;
using System.Linq;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples
{
    /// <summary>
    /// Contains utility methods used by samples in the <see cref="Extraction"/> namespace 
    /// to help perform extractions against the API.
    /// </summary>
    public static class ExtractionUtility
    {
        /// <summary>
        /// Validates the specified list of instrument identifiers to be used for an extraction.
        /// </summary>
        /// <param name="validationRequest">
        /// An <see cref="InstrumentValidationRequest"/> that contains the criteria of the validation.
        /// </param>
        /// <param name="proxy">
        /// The API proxy <see cref="ExtractionServiceClient"/> instance to use to perfom the validation.
        /// </param>
        /// <param name="credentials">
        /// A populated <see cref="CredentialsHeader"/> instance to pass to the API call.
        /// </param>
        /// <returns>
        /// An array of <see cref="ValidatedInstrument"/> objects from the API.
        /// </returns>
        public static ValidatedInstrument[] ValidateInstruments(InstrumentValidationRequest validationRequest, ExtractionServiceClient proxy, ref CredentialsHeader credentials)
        {
            // Create and initialize the request object. 
            var request = new ValidateInstrumentsRequest
                              {
                                  CredentialsHeader = credentials,
                                  request = validationRequest
                              };

            //call API using ApiUtility.Execute helper method
            var response = ApiUtility.Execute(
                () => proxy.ValidateInstruments(request));

            //update credentials reference so it can be resused by the caller
            credentials = response.CredentialsHeader;

            return response.ValidateInstrumentsResult.Instruments;
        }

		/// <summary>
		/// Validates the specified list of Legal Entity identifiers to be used for an extraction.
		/// </summary>
		/// <param name="validationRequest">
		/// An<see cref="LegalEntityValidationRequest"/> that contains the criteria of the validation.
		/// </param>
		/// <param name="proxy">
		/// The API proxy <see cref="ExtractionServiceClient"/> instance to use to perfom the validation.
		/// </param>
		/// <param name="credentials">
		/// A populated <see cref="CredentialsHeader"/> instance to pass to the API call.
		/// </param>
		/// <returns>
		/// An array of <see cref="ValidatedLegalEntity"/> objects from the API.
		/// </returns>
		public static ValidatedLegalEntity[] ValidateLegalEntities(LegalEntityValidationRequest validationRequest, ExtractionServiceClient proxy, ref CredentialsHeader credentials)
		{
			// Create and initialize the request object. 
			var request = new ValidateLegalEntitiesRequest
			{
				CredentialsHeader = credentials,
				request = validationRequest
			};

			//call API using ApiUtility.Execute helper method
			var response = ApiUtility.Execute(
				() => proxy.ValidateLegalEntities(request));

			//update credentials reference so it can be resused by the caller
			credentials = response.CredentialsHeader;

			return response.ValidateLegalEntitiesResult.LegalEntities;
		}

		/// <summary>
        /// Performs an instrument search, returning the first specified limit of found instruments for an extraction.
        /// </summary>
        /// <param name="searchRequest">
        /// A <see cref="InstrumentSearchRequest"/> that contains the criteria of the search.
        /// </param>
        /// <param name="proxy">
        /// The API proxy <see cref="ExtractionServiceClient"/> instance to use to perfom the validation.
        /// </param>
        /// <param name="credentials">
        /// A populated <see cref="CredentialsHeader"/> instance to pass to the API call.
        /// </param>
        /// <returns>
        /// An array of <see cref="ValidatedInstrument"/> objects of the first returned from the search.
        /// </returns>
        public static ValidatedInstrument[] SearchForInstruments(InstrumentSearchRequest searchRequest, ExtractionServiceClient proxy, ref CredentialsHeader credentials)
        {
            // Create and initialize the request object. 
            var request = new SearchInstrumentsRequest
                              {
                                  CredentialsHeader = credentials,
                                  request = searchRequest
                              };

            //call API using ApiUtility.Execute helper method
            var response = ApiUtility.Execute(
                () => proxy.SearchInstruments(request));

            //update credentials reference so it can be resused by the caller
            credentials = response.CredentialsHeader;

            //return only the specified amount
            return response.SearchInstrumentsResult.Instruments;
        }

        /// <summary>
        /// Batches the items in the specified list of instruments, which is useful for performing extractions 
        /// with the API when you have more instruments than you can send in a single extraction call.
        /// </summary>
        /// <param name="instruments">
        /// The source list of <see cref="ValidatedInstrument"/> objects.
        /// </param>
        /// <param name="batchSize">
        /// The size of each batch.
        /// </param>
        /// <returns>
        /// A list of batched <see cref="ValidatedInstrument"/> objects, batched as arrays so they can be easily passed to the API.
        /// </returns>
        public static IEnumerable<ValidatedInstrument[]> BatchInstruments(IEnumerable<ValidatedInstrument> instruments, int batchSize)
        {
            var batch = new List<ValidatedInstrument>();

            //loop through each instrument in source list
            foreach (var instrument in instruments)
            {
                //add instrument to current batch
                batch.Add(instrument);

                //keep adding to batch if its smaller than the max size
                if (batch.Count < batchSize) continue;

                //return filled batch
                yield return batch.ToArray();

                //create empty batch for next set of instruments
                batch = new List<ValidatedInstrument>();
            }

            //return final, partially-filled batch
            if (batch.Count > 0)
                yield return batch.ToArray();
        }

        /// <summary>
        /// Pivots the data conatined in the specified extraction columns into row-based tabular data.
        /// </summary>
        /// <param name="columns">
        /// The list of <see cref="ExtractionColumn"/> objects from an extraction result to convert.
        /// </param>
        /// <param name="rowCount">
        /// The row count of the resulting data.
        /// </param>
        /// <returns>
        /// A <see cref="RowBasedResult"/> object containing the row-based data.
        /// </returns>
        public static RowBasedResult PivotExtractionResult(IEnumerable<ExtractionColumn> columns, int rowCount)
        {
            columns = columns ?? new ExtractionColumn[]{};
            var rows = new List<object[]>();

            for (var i = 0; i < rowCount; i++)
            {
                var index = i;
                var rowValues = (from c in columns select c.Values[index]).ToArray();

                rows.Add(rowValues);
            }

            return new RowBasedResult
                       {
                           ColumnNames = (from c in columns select c.Name).ToArray(),
                           Rows = rows
                       };
        }

        /// <summary>
        /// Pivots the data conatined in the specified usage columns into row-based tabular data.
        /// </summary>
        /// <param name="columns">
        /// The list of <see cref="UsageColumn"/> objects from an usage result to convert.
        /// </param>
        /// <param name="rowCount">
        /// The row count of the resulting data.
        /// </param>
        /// <returns>
        /// A <see cref="RowBasedResult"/> object containing the row-based data.
        /// </returns>
        public static RowBasedResult PivotUsageResult(IEnumerable<UsageColumn> columns, int rowCount)
        {
            var rows = new List<object[]>();
            for (var i = 0; i < rowCount; i++)
            {
                var index = i;
                var rowValues = (from c in columns select c.Values[index]).ToArray();

                rows.Add(rowValues);
            }

            return new RowBasedResult
            {
                ColumnNames = (from c in columns select c.Name).ToArray(),
                Rows = rows
            };
        }

        /// <summary>
        /// Confirms that all output fields specified in an extraction were supported by the API.
        /// </summary>
        /// <param name="extractionResponse">
        /// The <see cref="ExtractionResponse"/> from the API after an extraction was performed.
        /// </param>
        public static void EnsureOutputFields(ExtractionResponse extractionResponse)
        {
            var invalidOutputFields = (from c in extractionResponse.Columns
                                       where c.FromInvalidOutputField
                                       select c.Name).ToList();
            if (invalidOutputFields.Count > 0)
                throw new InvalidOutputFieldsException(
                    "There were one or more output fields requested in the extraction that were not supported.",
                    invalidOutputFields);
        }
    }
}
