using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Search.Counterparty
{
    public class AllSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string identfierType = "RCP";
                    const string searchValue = "30001228*";

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Searching for all Counterparties that:");
                    writer.WriteLine("- are {0} identifiers that begin with {1}", identfierType, searchValue);

                    #endregion

                    #region Search counterparties

                    // Create and initialize the request object. 
                    var request =
                        new SearchCounterpartiesRequest
                            {
                                CredentialsHeader = credentials,
                                request = new CounterpartySearchRequestAll
                                              {
                                                  // These are required values.
                                                  // NOTE: call GetCounterpartyIdentifierTypes operation to all Counterparty identifier types.
                                                  IdentifierType = identfierType,
                                                  IdentifierSearchValue = searchValue
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    SearchCounterpartiesResponse response;
                    try
                    {
                        response = proxy.SearchCounterparties(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    // Display the response.
                    writer.WriteLine("{0} Counterparties found:", response.SearchCounterpartiesResult.Counterparties.Length);
                    foreach (var counterparty in response.SearchCounterpartiesResult.Counterparties)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     counterparty.IdentifierType,
                                     counterparty.IdentifierValue);
                        writer.WriteLine("  - RCP ID = {0}", counterparty.RcpId);
                        writer.WriteLine("  - Counterparty ID = {0}", counterparty.CounterpartyId);
                        writer.WriteLine("  - Description = {0}", counterparty.Description);
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}