﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Search.Instrument
{
    public class FundsSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string currencyType = "GBP"; //British Pound
                    const string domicileType = "VG"; // Virgin Islands (British)

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Searching for Funds Instruments that:");
                    writer.WriteLine("- use a currency type of ({0})", currencyType);
                    writer.WriteLine("- have a domicile type of ({0})", domicileType);

                    #endregion

                    #region Search instruments

                    // Create and initialize the request object. 
                    var request =
                        new SearchInstrumentsRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentSearchRequestFunds
                                              {
                                                  // These are required values.
                                                  // NOTE: call GetInstrumentSearchTypes operation to see all identifier types supported by this search type.
                                                  IdentifierType = "RIC",
                                                  IdentifierSearchValue = "*",
                                                  PreferredIdentifierType = "RIC",
                                                  // NOTE: call the GetFundsCurrencyTypes operation to see all instrument sub types.
                                                  CurrencyType = currencyType,
                                                  // NOTE: call the GetFundsDomicileTypes operation to see all instrument sub types.
                                                  DomicileType = domicileType
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    SearchInstrumentsResponse response;
                    try
                    {
                        response = proxy.SearchInstruments(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("{0} Instruments found:", response.SearchInstrumentsResult.Instruments.Length);
                    foreach (var instrument in response.SearchInstrumentsResult.Instruments)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     instrument.IdentifierType,
                                     instrument.IdentifierValue);
                        writer.WriteLine("  - Asset ID = {0}", instrument.AssetId);
                        writer.WriteLine("  - Quote ID = {0}", instrument.QuoteId);
                        writer.WriteLine("  - Source = {0}", instrument.Source);
                        writer.WriteLine("  - RIC = {0}", instrument.Ric);
                        writer.WriteLine("  - File Code = {0}", instrument.FileCode);
                        writer.WriteLine("  - Object Type = {0}", instrument.ObjectType);
                        writer.WriteLine("  - Segment = {0}", instrument.Segment);
                        writer.WriteLine("  - Description = {0}", instrument.Description);

                        if (instrument.ValidatedLegalEntity == null) continue;

						var legalEntity = instrument.ValidatedLegalEntity;
						writer.WriteLine("  Legal Entity = {0}: {1}",
									 legalEntity.IdentifierType,
									 legalEntity.IdentifierValue);
						writer.WriteLine("    - Org ID = {0}", legalEntity.OrgId);
						writer.WriteLine("    - Entity ID = {0}", legalEntity.EntityId);
						writer.WriteLine("    - Description = {0}", legalEntity.Description);
					}

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}