﻿using System;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Search.Instrument
{
    public class FuturesAndOptionsSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string exchangeType = "MGE";
                    const string putCallType = "P";
                    const string description = "1II*";
                    var thisYear = DateTime.Now.Year;
                    var fromDate = new DateTime(thisYear, 1, 1);
                    var toDate = new DateTime(thisYear, 12, 31);
                    const int strikePriceLimit = 1000;

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Searching for Futures and Options Instruments that:");
                    writer.WriteLine("- are traded in the Minneapolis Grain Exchange ({0})", exchangeType);
                    writer.WriteLine("- are puts ({0})", putCallType);
                    writer.WriteLine("- have a description that starts with {0}", description);
                    writer.WriteLine("- expire this year ({0:d} - {1:d})", fromDate, toDate);
                    writer.WriteLine("- have a strike price that is greater than {0}", strikePriceLimit);

                    #endregion

                    #region Search instruments

                    // Create and initialize the request object.
                    var request =
                        new SearchInstrumentsRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentSearchRequestFuturesAndOptions
                                              {
                                                  // These are required values.
                                                  // NOTE: call GetInstrumentSearchTypes operation to see all identifier types supported by this search type.
                                                  IdentifierType = "RIC",
                                                  IdentifierSearchValue = "*",
                                                  PreferredIdentifierType = "RIC",
                                                  // NOTE: call GetFuturesAndOptionsExchangeTypes operation to see all exchange types.
                                                  ExchangeTypes = new[] {exchangeType},
                                                  // NOTE: call GetFuturesAndOptionsPutCallTypes operation to see all put/call types.
                                                  PutCallType = putCallType,
                                                  Description = description,
                                                  // NOTE: call GetComparisonOperators operation to see all comparison operators.
                                                  ExpirationDateComparisonOperator = "FROM",
                                                  ExpirationDatePrimaryValue = fromDate,
                                                  ExpirationDateSecondaryValue = toDate,
                                                  // NOTE: call GetComparisonOperators operation to see all comparison operators.
                                                  StrikePriceComparisonOperator = ">",
                                                  StrikePricePrimaryValue = 1000
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    SearchInstrumentsResponse response;
                    try
                    {
                        response = proxy.SearchInstruments(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("{0} Instruments found:", response.SearchInstrumentsResult.Instruments.Length);
                    foreach (var instrument in response.SearchInstrumentsResult.Instruments)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     instrument.IdentifierType,
                                     instrument.IdentifierValue);
                        writer.WriteLine("  - Asset ID = {0}", instrument.AssetId);
                        writer.WriteLine("  - Quote ID = {0}", instrument.QuoteId);
                        writer.WriteLine("  - Source = {0}", instrument.Source);
                        writer.WriteLine("  - RIC = {0}", instrument.Ric);
                        writer.WriteLine("  - File Code = {0}", instrument.FileCode);
                        writer.WriteLine("  - Object Type = {0}", instrument.ObjectType);
                        writer.WriteLine("  - Segment = {0}", instrument.Segment);

						if (instrument.ValidatedLegalEntity == null) continue;

						var legalEntity = instrument.ValidatedLegalEntity;
						writer.WriteLine("  Legal Entity = {0}: {1}",
									 legalEntity.IdentifierType,
									 legalEntity.IdentifierValue);
						writer.WriteLine("    - Org ID = {0}", legalEntity.OrgId);
						writer.WriteLine("    - Entity ID = {0}", legalEntity.EntityId);
						writer.WriteLine("    - Description = {0}", legalEntity.Description);
					}

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}