﻿using System;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Search.Instrument
{
    public class GovCorpSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly.
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string country = "EU";
                    const string moodyType = "Aa1";
                    const string spType = "AA+";
                    const string subGroup = "GBND";
                    var thisYear = DateTime.Now.Year;
                    var fromDate = new DateTime(thisYear, 1, 1);
                    var toDate = new DateTime(thisYear, 12, 31);
                    const int couponLimit = 1;

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Searching for Gov/Corp Instruments that:");
                    writer.WriteLine("- are government bonds");
                    writer.WriteLine("- have a sub-group type of 'Notes/Bonds Govt' ({0})", subGroup);
                    writer.WriteLine("- have a Moody's rating of {0}", moodyType);
                    writer.WriteLine("- have a S&P rating of {0}", spType);
                    writer.WriteLine("- are from the European Union ({0})", country);
                    writer.WriteLine("- mature this year ({0:d} - {1:d})", fromDate, toDate);
                    writer.WriteLine("- have a coupon value that is greater than {0}", couponLimit);

                    #endregion

                    #region Search instruments

                    // Create and initialize the request object.
                    var request =
                        new SearchInstrumentsRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentSearchRequestGovCorp
                                              {
                                                  // These are required values.
                                                  IdentifierType = "CSP",
                                                  IdentifierSearchValue = "*",
                                                  PreferredIdentifierType = "CSP",
                                                  IsGovernmentGroup = true,
                                                  // NOTE: call the GetGovCorpSubGroupTypes method to see a list of allowable subgroup types.
                                                  SubGroupType = subGroup,
                                                  // NOTE: call the GetGovCorpMoodyTypes method to see a list of allowable Moody rating types.
                                                  MoodyTypes = new[] {moodyType},
                                                  // NOTE: call the GetGovCorpStandartPoorsTypes method to see a list of allowable Standard and Poors rating types.
                                                  StandardPoorsTypes = new[] {spType},
                                                  // NOTE: call the GetGovCorpCountryTypes method to see a list of allowable country types.
                                                  CountryType = country,
                                                  // NOTE: call the GetComparisonOperators method to see a list of allowable comparison operators.
                                                  MaturityDateComparisonOperator = "FROM",
                                                  MaturityDatePrimaryValue = fromDate,
                                                  MaturityDateSecondaryValue = toDate,
                                                  // NOTE: call the GetComparisonOperators method to see a list of allowable comparison operators.
                                                  CouponComparisonOperator = ">",
                                                  // Set the CouponPrimaryValue member to the integer couponLimit.
                                                  CouponPrimaryValue = couponLimit
                                              }
                            };

                    SearchInstrumentsResponse response;
                    try
                    {
                        response = proxy.SearchInstruments(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("{0} Instruments found:", response.SearchInstrumentsResult.Instruments.Length);
                    foreach (var instrument in response.SearchInstrumentsResult.Instruments)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     instrument.IdentifierType,
                                     instrument.IdentifierValue);
                        writer.WriteLine("  - Asset ID = {0}", instrument.AssetId);
                        writer.WriteLine("  - Quote ID = {0}", instrument.QuoteId);
                        writer.WriteLine("  - Source = {0}", instrument.Source);
                        writer.WriteLine("  - RIC = {0}", instrument.Ric);
                        writer.WriteLine("  - File Code = {0}", instrument.FileCode);
                        writer.WriteLine("  - Object Type = {0}", instrument.ObjectType);
                        writer.WriteLine("  - Segment = {0}", instrument.Segment);

						if (instrument.ValidatedLegalEntity == null) continue;

						var legalEntity = instrument.ValidatedLegalEntity;
						writer.WriteLine("  Legal Entity = {0}: {1}",
									 legalEntity.IdentifierType,
									 legalEntity.IdentifierValue);
						writer.WriteLine("    - Org ID = {0}", legalEntity.OrgId);
						writer.WriteLine("    - Entity ID = {0}", legalEntity.EntityId);
						writer.WriteLine("    - Description = {0}", legalEntity.Description);
					}

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}