﻿using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Search.Instrument
{
    public class MortPassThruSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string agency = "0x000019000050662f";
                    const string amoritizationType = "A";
                    const string assetStatus = "MAT";
                    const int couponLimit = 2;

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Searching for Mort Pass Thru Instruments that:");
                    writer.WriteLine("- are handled by Fannie Mae ({0})", agency);
                    writer.WriteLine("- are ARM mortgages ({0})", amoritizationType);
                    writer.WriteLine("- have matured ({0})", assetStatus);
                    writer.WriteLine("- have a net coupon value that is less than {0}", couponLimit);

                    #endregion

                    #region Search instruments

                    // Create and initialize the request object.
                    var request =
                        new SearchInstrumentsRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentSearchRequestMortPassThru
                                              {
                                                  // These are required values.
                                                  // NOTE: call GetInstrumentSearchTypes operation to see all identifier types supported by this search type.
                                                  IdentifierType = "CSP",
                                                  IdentifierSearchValue = "*",
                                                  PreferredIdentifierType = "CSP",
                                                  // NOTE: call GetMbsAgencyTypes operation to see all agency types.
                                                  AgencyType = agency,
                                                  // NOTE: call GetMbsAmortizationTypes operation to see all agency types.
                                                  AmortizationType = amoritizationType,
                                                  // NOTE: call GetMbsAssetStatusTypes operation to see all status types.
                                                  AssetStatusTypes = new[] {assetStatus},
                                                  // NOTE: call GetComparisonOperators operation to see all comparison operators.
                                                  NetCouponComparisonOperator = "<",
                                                  NetCouponPrimaryValue = couponLimit
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    SearchInstrumentsResponse response;
                    try
                    {
                        response = proxy.SearchInstruments(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("{0} Instruments found:", response.SearchInstrumentsResult.Instruments.Length);
                    foreach (var instrument in response.SearchInstrumentsResult.Instruments)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     instrument.IdentifierType,
                                     instrument.IdentifierValue);
                        writer.WriteLine("  - Asset ID = {0}", instrument.AssetId);
                        writer.WriteLine("  - Quote ID = {0}", instrument.QuoteId);
                        writer.WriteLine("  - Source = {0}", instrument.Source);
                        writer.WriteLine("  - RIC = {0}", instrument.Ric);
                        writer.WriteLine("  - File Code = {0}", instrument.FileCode);
                        writer.WriteLine("  - Object Type = {0}", instrument.ObjectType);
                        writer.WriteLine("  - Segment = {0}", instrument.Segment);

						if (instrument.ValidatedLegalEntity == null) continue;

						var legalEntity = instrument.ValidatedLegalEntity;
						writer.WriteLine("  Legal Entity = {0}: {1}",
									 legalEntity.IdentifierType,
									 legalEntity.IdentifierValue);
						writer.WriteLine("    - Org ID = {0}", legalEntity.OrgId);
						writer.WriteLine("    - Entity ID = {0}", legalEntity.EntityId);
						writer.WriteLine("    - Description = {0}", legalEntity.Description);
					}

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}