﻿using System;
using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Search.Instrument
{
    public class UsMunicipalsSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    const string spRating = "AA+";
                    const string state = "NY";
                    var thisYear = DateTime.Now.Year;
                    var fromDate = new DateTime(thisYear, 1, 1);
                    var toDate = new DateTime(thisYear, 12, 31);
                    const int couponLimit = 5;

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Searching for US Municpial Instruments that:");
                    writer.WriteLine("- have a S&P rating of {0}", spRating);
                    writer.WriteLine("- are from New York ({0})", state);
                    writer.WriteLine("- are callable");
                    writer.WriteLine("- mature this year ({0:d} - {1:d})", fromDate, toDate);
                    writer.WriteLine("- have a coupon value that is less than {0}", couponLimit);

                    #endregion

                    #region Search instruments

                    // Create and initialize the request object.
                    var request =
                        new SearchInstrumentsRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentSearchRequestUSMunicipals
                                              {
                                                  // These are required values.
                                                  // NOTE: call GetInstrumentSearchTypes operation to see all identifier types supported by this search type.
                                                  IdentifierType = "ISN",
                                                  IdentifierSearchValue = "*",
                                                  PreferredIdentifierType = "ISN",
                                                  // NOTE: call GetUSMunicipalsStandardPoorsTypes operation to see all S&P types.
                                                  StandardPoorsTypes = new[] { spRating },
                                                  // NOTE: call GetUSMunicipalsStateTypes operation to see all S&P types.
                                                  StateType = state,
                                                  IsCallable = true,
                                                  // NOTE: call GetComparisonOperators operation to see all comparison operators.
                                                  MaturityDateComparisonOperator = "FROM",
                                                  MaturityDatePrimaryValue = fromDate,
                                                  MaturityDateSecondaryValue = toDate,
                                                  // NOTE: call GetComparisonOperators operation to see all comparison operators.
                                                  CouponComparisonOperator = "<",
                                                  CouponPrimaryValue = couponLimit
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    SearchInstrumentsResponse response;
                    try
                    {
                        response = proxy.SearchInstruments(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("{0} Instruments found:", response.SearchInstrumentsResult.Instruments.Length);
                    foreach (var instrument in response.SearchInstrumentsResult.Instruments)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     instrument.IdentifierType,
                                     instrument.IdentifierValue);
                        writer.WriteLine("  - Asset ID = {0}", instrument.AssetId);
                        writer.WriteLine("  - Quote ID = {0}", instrument.QuoteId);
                        writer.WriteLine("  - Source = {0}", instrument.Source);
                        writer.WriteLine("  - RIC = {0}", instrument.Ric);
                        writer.WriteLine("  - File Code = {0}", instrument.FileCode);
                        writer.WriteLine("  - Object Type = {0}", instrument.ObjectType);
                        writer.WriteLine("  - Segment = {0}", instrument.Segment);

						if (instrument.ValidatedLegalEntity == null) continue;

						var legalEntity = instrument.ValidatedLegalEntity;
						writer.WriteLine("  Legal Entity = {0}: {1}",
									 legalEntity.IdentifierType,
									 legalEntity.IdentifierValue);
						writer.WriteLine("    - Org ID = {0}", legalEntity.OrgId);
						writer.WriteLine("    - Entity ID = {0}", legalEntity.EntityId);
						writer.WriteLine("    - Description = {0}", legalEntity.Description);
					}

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}