﻿using System;
using System.ServiceModel;
using DssApiSamples.Samples.Extraction;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Usage
{
    public class ExtractionUsageLegalEntitySummarySample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var startDateTime = DateTime.Now.AddMonths(-1);
                    var endDateTime = DateTime.Now;

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Reporting usage for user:");
                    writer.WriteLine("- for extracted counterparties between dates: {0} and {1}",
                                 startDateTime.ToLongDateString(), endDateTime.ToLongDateString());

                    #endregion

                    #region Get Usage data

                    writer.WriteLine("Extracting usage data ...");

                    // Create and initialize the request object. 
                    var request =
                        new ReportUsageRequest
                            {
                                CredentialsHeader = credentials,
                                request = new  ExtractionUsageLegalEntitySummaryRequest
                                              {
                                                  StartDateTime = startDateTime,
                                                  EndDateTime = endDateTime
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    ReportUsageResponse response;
                    try
                    {
                        response = proxy.ReportUsage(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    var usageResponse = response.ReportUsageResult;
                    writer.WriteLine("Row Count = {0}", usageResponse.RowCount);

                    // Pivot the column-based results from API to row-based results.
                    var rowBasedResults = ExtractionUtility.PivotUsageResult(usageResponse.Columns,
                                                                             usageResponse.RowCount);

                    // Display the results in CSV format.
                    writer.WriteLine(
                        "Row Data (in CSV format - copy/paste into new file to open in a spreadsheet application) =");
                    writer.WriteLine();
                    writer.WriteLine(ApiUtility.ToCsvLine(rowBasedResults.ColumnNames));
                    foreach (var row in rowBasedResults.Rows)
                        writer.WriteLine(ApiUtility.ToCsvLine(row));
                    writer.WriteLine();

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}