using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Validation
{
    public class CounterpartySample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                                          {
                                              new CounterpartyIdentifier
                                                  {
                                                      // NOTE: call GetCounterpartyIdentifierTypes operation to see all instrument identifier types.
                                                      IdentifierType = "RCP",
                                                      IdentifierValue = "300012286"
                                                  },
                                              new CounterpartyIdentifier
                                                  {
                                                      IdentifierType = "RCP",
                                                      IdentifierValue = "INVALID-VALUE"
                                                  }
                                          };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Validating Counterparty identifiers:");
                    foreach (var identifier in identifiers)
                        writer.WriteLine("  {0}: {1}", identifier.IdentifierType, identifier.IdentifierValue);

                    #endregion

                    #region Perform validation

                    // Create and initialize the request object. 
                    var request =
                        new ValidateCounterpartiesRequest
                            {
                                CredentialsHeader = credentials,
                                request = new CounterpartyValidationRequest
                                              {
                                                  Identifiers = identifiers
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    ValidateCounterpartiesResponse response;
                    try
                    {
                        response = proxy.ValidateCounterparties(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("Validated Counterparties:");
                    foreach (var counterparty in response.ValidateCounterpartiesResult.Counterparties)
                    {
                        writer.WriteLine("  {0}: {1}", counterparty.IdentifierType, counterparty.IdentifierValue);
                        writer.WriteLine("  - RCP ID = {0}", counterparty.RcpId);
                        writer.WriteLine("  - Counterparty ID = {0}", counterparty.CounterpartyId);
                        writer.WriteLine("  - Description = {0}", counterparty.Description);
                    }

                    writer.WriteLine("Unmapped identifiers:");
                    foreach (var identifier in response.ValidateCounterpartiesResult.UnmappedIdentifiers)
                        writer.WriteLine("  {0}: {1}", identifier.IdentifierType, identifier.IdentifierValue);

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}