using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Validation
{
    public class InstrumentSample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                                          {
                                              new InstrumentIdentifier
                                                  {
                                                      // NOTE: call GetInstrumentIdentifierTypes operation to see all instrument identifier types.
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "LCOc1"
                                                  },
                                              new InstrumentIdentifier
                                                  {
                                                      IdentifierType = "RIC",
                                                      IdentifierValue = "INVALID-VALUE"
                                                  }
                                          };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Validating Instrument identifiers:");
                    foreach (var identifier in identifiers)
                        writer.WriteLine("  {0}: {1}", identifier.IdentifierType, identifier.IdentifierValue);

                    // Create and initialize the request object. 
                    var request =
                        new ValidateInstrumentsRequest
                            {
                                CredentialsHeader = credentials,
                                request = new InstrumentValidationRequest
                                              {
                                                  Identifiers = identifiers
                                              }
                            };

                    #endregion

                    #region Perform validation

                    // Call the API to get the response, handling API errors.
                    ValidateInstrumentsResponse response;
                    try
                    {
                        response = proxy.ValidateInstruments(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("Validated Instruments:");
                    foreach (var instrument in response.ValidateInstrumentsResult.Instruments)
                    {
                        writer.WriteLine("  {0}: {1}",
                                     instrument.IdentifierType,
                                     instrument.IdentifierValue);
                        writer.WriteLine("  - Asset ID = {0}", instrument.AssetId);
                        writer.WriteLine("  - Quote ID = {0}", instrument.QuoteId);
                        writer.WriteLine("  - Source = {0}", instrument.Source);
                        writer.WriteLine("  - RIC = {0}", instrument.Ric);
                        writer.WriteLine("  - File Code = {0}", instrument.FileCode);
                        writer.WriteLine("  - Object Type = {0}", instrument.ObjectType);
                        writer.WriteLine("  - Segment = {0}", instrument.Segment);
                    }

                    writer.WriteLine("Unmapped identifiers:");
                    for (var i = 0; i < response.ValidateInstrumentsResult.UnmappedIdentifiers.Length; i++)
                    {
                        var error = response.ValidateInstrumentsResult.UnmappedIdentifierErrors[i];
                        writer.WriteLine("  " + error + ":");
                        var identifier = response.ValidateInstrumentsResult.UnmappedIdentifiers[i];
                        writer.WriteLine("  {0}: {1} {2}",
                                     identifier.IdentifierType,
                                     identifier.IdentifierValue,
                                     string.IsNullOrEmpty(identifier.Source) ? null : "(" + identifier.Source + ")");
                    }

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}