using System.ServiceModel;
using DssApiSamples.Core;
using DssApiSamples.Samples.Service;

namespace DssApiSamples.Samples.Validation
{
    public class LegalEntitySample : ISample
    {
        public void Execute(IMessageWriter writer)
        {
            // The "using" statement ensures the proxy is properly disposed if no exceptions are thrown.
            using (var proxy = new ExtractionServiceClient())
            {
                // Wrap the proxy call in a try block in order to handle communication exceptions properly. 
                try
                {
                    // Credentials are required to authenticate the request.
                    var credentials = ApiUtility.CreateCredentials();

                    #region Declare inputs

                    var identifiers = new[]
                                          {
                                              new LegalEntityIdentifier
                                                  {
                                                      // NOTE: call GetLegalEntityIdentifierTypes operation to see all legal entity identifier types.
                                                      IdentifierType = "ORG",
                                                      IdentifierValue = "12286"
                                                  },
                                              new LegalEntityIdentifier
                                                  {
                                                      IdentifierType = "ORG",
                                                      IdentifierValue = "INVALID-VALUE"
                                                  }
                                          };

                    #endregion

                    #region Display sample description

                    writer.WriteLine("Validating Legal Entity identifiers:");
                    foreach (var identifier in identifiers)
                        writer.WriteLine("  {0}: {1}", identifier.IdentifierType, identifier.IdentifierValue);

                    #endregion

                    #region Perform validation

                    // Create and initialize the request object. 
                    var request =
                        new ValidateLegalEntitiesRequest
                            {
                                CredentialsHeader = credentials,
                                request = new LegalEntityValidationRequest
                                              {
                                                  Identifiers = identifiers
                                              }
                            };

                    // Call the API to get the response, handling API errors.
                    ValidateLegalEntitiesResponse response;
                    try
                    {
                        response = proxy.ValidateLegalEntities(request);
                    }
                    catch (FaultException ex)
                    {
                        throw ApiUtility.ConvertFaultException(ex);
                    }

                    #endregion

                    #region Display results

                    writer.WriteLine("Validated Legal Entities:");
                    foreach (var legalEntity in response.ValidateLegalEntitiesResult.LegalEntities)
                    {
                        writer.WriteLine("  {0}: {1}", legalEntity.IdentifierType, legalEntity.IdentifierValue);
                        writer.WriteLine("  - ORG ID = {0}", legalEntity.OrgId);
                        writer.WriteLine("  - Entity ID = {0}", legalEntity.EntityId);
                        writer.WriteLine("  - Description = {0}", legalEntity.Description);
                    }

                    writer.WriteLine("Unmapped identifiers:");
                    foreach (var identifier in response.ValidateLegalEntitiesResult.UnmappedIdentifiers)
                        writer.WriteLine("  {0}: {1}", identifier.IdentifierType, identifier.IdentifierValue);

                    #endregion
                }
                // If any exception occurs, close the communication channel and rethrow the exception.
                catch
                {
                    proxy.Abort();
                    throw;
                }
            }
        }
    }
}